@@ -134,8 +134,8 @@ static const EffectToPipelineDesc EFFECT_TO_PIPELINE_DESC[] = {
 
 static const int NUM_EFFECTS = G_N_ELEMENTS (EFFECT_TO_PIPELINE_DESC);
 
-GST_DEBUG_CATEGORY (cheese_camera);
-#define GST_CAT_DEFAULT cheese_camera
+GST_DEBUG_CATEGORY (cheese_camera_cat);
+#define GST_CAT_DEFAULT cheese_camera_cat
 
 GQuark
 cheese_camera_error_quark (void)
@@ -223,6 +223,7 @@ cheese_camera_photo_data_cb (GstElement *element, GstBuffer *buffer,
   int                 width, height, stride;
   GdkPixbuf          *pixbuf;
   const int           bits_per_pixel = 8;
+  guchar             *data;
 
   caps      = gst_buffer_get_caps (buffer);
   structure = gst_caps_get_structure (caps, 0);
@@ -230,9 +231,17 @@ cheese_camera_photo_data_cb (GstElement *element, GstBuffer *buffer,
   gst_structure_get_int (structure, "height", &height);
 
   stride = buffer->size / height;
-  pixbuf = gdk_pixbuf_new_from_data (GST_BUFFER_DATA (buffer), GDK_COLORSPACE_RGB,
+
+  /* Only copy the data if we're giving away a pixbuf,
+   * not if we're throwing everything away straight away */
+  if (priv->photo_filename != NULL)
+    data = NULL;
+  else
+    data = g_memdup (GST_BUFFER_DATA (buffer), buffer->size);
+  pixbuf = gdk_pixbuf_new_from_data (data ? data : GST_BUFFER_DATA (buffer),
+                                     GDK_COLORSPACE_RGB,
                                      FALSE, bits_per_pixel, width, height, stride,
-                                     NULL, NULL);
+                                     data ? (GdkPixbufDestroyNotify) g_free : NULL, NULL);
 
   g_signal_handler_disconnect (G_OBJECT (priv->photo_sink),
                                priv->photo_handler_signal_id);
@@ -279,13 +288,28 @@ cheese_camera_bus_message_cb (GstBus *bus, GstMessage *message, CheeseCamera *ca
 
 static void
 cheese_camera_add_device (CheeseCameraDeviceMonitor *monitor,
-                          CheeseCameraDevice        *device,
+                          const gchar               *id,
+                          const gchar               *device_file,
+                          const gchar               *product_name,
+                          gint                       api_version,
                           CheeseCamera              *camera)
 {
   CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
+  GError *error;
 
-  g_ptr_array_add (priv->camera_devices, device);
-  priv->num_camera_devices++;
+  CheeseCameraDevice *device = cheese_camera_device_new (id,
+                                                         device_file,
+                                                         product_name,
+                                                         api_version,
+                                                         &error);
+  if (device == NULL)
+    GST_WARNING ("Device initialization for %s failed: %s ",
+                 device_file,
+                 (error != NULL) ? error->message : "Unknown reason");
+  else  {
+    g_ptr_array_add (priv->camera_devices, device);
+    priv->num_camera_devices++;
+  }
 }
 
 static void
@@ -1002,6 +1026,11 @@ cheese_camera_class_init (CheeseCameraClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  if (cheese_camera_cat == NULL)
+    GST_DEBUG_CATEGORY_INIT (cheese_camera_cat,
+                             "cheese-camera",
+                             0, "Cheese Camera");
+
   object_class->finalize     = cheese_camera_finalize;
   object_class->get_property = cheese_camera_get_property;
   object_class->set_property = cheese_camera_set_property;
@@ -1056,10 +1085,6 @@ cheese_camera_init (CheeseCamera *camera)
 {
   CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
 
-  GST_DEBUG_CATEGORY_INIT (cheese_camera,
-                           "cheese-camera",
-                           0, "Cheese Camera");
-
   priv->is_recording            = FALSE;
   priv->pipeline_is_playing     = FALSE;
   priv->photo_filename          = NULL;
